#include <allegro.h>
#ifdef WIN32
#include <winalleg.h>
#endif

#include "log.h"
#include "bitmap.h"
#include "conf.h"
#include "sounds.h"
#include "particle.h"
#include "print.h"
#include "datafile.h"
#include "byteorder.h"
#include "guimanager.h"
#include "guitilemap.h"
#include "settings.h"
#include "world.h"
#include "socket.h" // Must include winalleg before this file, if on windows
#include <string>
#include <cctype>
using namespace std;

#ifndef ENGINE_NAME
#define ENGINE_NAME "Engine"
#endif

static volatile unsigned int msecs = 0;

static void increment()
{
    msecs += 20;
}
END_OF_FUNCTION(increment)

int main(int, char **)
{
    int ret = 0;
    
#if defined(OSX) && !defined(DEBUGMODE)
    
    string homedir = getenv("HOME");
    
    system(("mkdir " + homedir + "/Library/Logs/" ENGINE_NAME " 2> /dev/null").c_str());
    
    out.addOutFile((homedir + "/Library/Logs/" ENGINE_NAME "/log.txt").c_str());
    err.addOutFile((homedir + "/Library/Logs/" ENGINE_NAME "/errors.txt").c_str());
    out.removeOutStream(cout);
    err.removeOutStream(cerr);

    chdir("Contents/Resources");
    
    settings.filename = homedir + "/Library/Preferences/" ENGINE_NAME "prefs";
#else
    out.addOutFile("log.txt");
    err.addOutFile("errors.txt");
#endif
    
    out << ENGINE_NAME " starting.\n";
    
    bool loaded = 0;
    
    try {
        
        if(0 != allegro_init())
            throw "Unable to init Allegro.";
        
        srand(time(0));
        animation::msecs = &msecs;
        
        datafile.set_filename("data.dat");
        
        settings.load();
        
        Conf conf(Conf::WINDOWED, Conf::SOFTWARE);
        
        settings.apply();
        
        LOCK_VARIABLE(msecs)
        LOCK_FUNCTION(increment)
        
        install_int_ex(increment, MSEC_TO_TIMER(20));
        
        world.itemDb.load("world.idb");
        world.resourceDb.load("world.rdb");
        world.buildingDb.load("world.bdb");
        world.trainDb.load("world.tdb");
        world.loadCities("world.cit");
        world.loadResources("world.res");
        world.loadTracks("world.trk");
        world.loadTrains("world.trn");
        world.loadTerminals("world.trl");
        world.loadUsers("world.usr");
        loaded = 1;
        
        out << "This is a " << (endian::isLittle() ? "little" : "big") << " endian machine.\n";
        
        {
            GuiManager guiManager;
            Gui gui(&guiManager);
        }
    }
    catch(const char *error) {
        
        Conf::handleCaughtError(error);
        
        ret = 1;
    }
    
    if(loaded) {
        
        world.saveCities("world.cit");
        world.saveResources("world.res");
        world.saveTracks("world.trk");
        world.saveTrains("world.trn");
        world.saveTerminals("world.trl");
        world.saveUsers("world.usr");
    }
    
    settings.save();
    
    Particle::cleanup();
    
    out << ENGINE_NAME " shutdown.\n";
    
    return ret;
}
END_OF_MAIN()
