#ifndef CONF_H
#define CONF_H

#include <string>

// Sets up Allegro and AllegroGL
class Conf {
public:
    
    typedef int Flags;
    
    static const int WINDOWED;
    static const int FULLSCREEN;
    
    typedef int Mode;
    
    static const int SOFTWARE; /* Do rendering in software */
    static const int HARDWARE; /* Do rendering in hardware (gfx card) */
    
    // Throws a const char* on error
    Conf(Flags flags, Mode renderingMode) throw (const char*);
    ~Conf();
    
    bool changeResolution(Flags flags, int width, int height, int depth);
    
    // Should be called after done drawing in order to flip the buffers
    void flip();
    
    // Clears the buffer to be drawn to
    void clearScreen();
    
    // True if our program currently has the focus
    bool switchedIn();
    
    // Returns true if the program has gained focus since the last call to
    // switchedInAsync
    bool switchedInAsync();
    
    // Returns true if the program has lost focus since the last call to
    // switchedOutAsync
    bool switchedOutAsync();

    // Returns true if the user has requested to quit.
    bool requestedQuitAsync();
    
    // Returns the rendering mode
    Mode getRenderingMode();
    
    // This function should be called with whatever const char* has been
    // thrown by a method or ctor of Conf.  Will attempt to output
    // the error to the user in the best method possible.
    static void handleCaughtError(const std::string &error);

private:
    
    Mode renderingMode;
    
    void initSoftware(Flags);
    void flipSoftware();
    void clearScreenSoftware();
    bool changeResolutionSoftware(Flags flags, int width, int height, int depth);
    static void shutdownSoftware();
    
    void initHardware(Flags);
    void flipHardware();
    void clearScreenHardware();
    bool changeResolutionHardware(Flags flags, int width, int height, int depth);
    static void shutdownHardware();
};

extern Conf *globalConfInstance;

#endif
