#ifndef EMITMANAGER_H
#define EMITMANAGER_H

#include "emitter.h"
#include <list>

class Circles;

class EmitManager {
public:
    
    // Deletes all remaining emitters and particles
    ~EmitManager();
    
    // Adds Emitter e to the list of emitters.  e will be deleted by
    // EmitManager so the user must take care that he does not do so.
    Emitter &add(int program, int x, int y);
    
    // Calls the update method on all stored emitters and deletes emitters
    // that need deletion.  Also updates all store particles and deletes
    // ones that need deletion.
    void update();
    
    // Calls the draw method on all stored particles
    void draw(const Tint &screenTint = Tint());
    
    // Adds a particle to the internal list of particles and returns a reference to it.
    // objname specifies the name of its texture in the datafile and the second
    // parameter is the datafile where the object can be found.  Defaults to
    // the global datafile.
    Particle *addParticle(const std::string &objname, datafile_t* = &datafile);
    
    // Deletes all emitters.
    void clearEmitters();
    
    // Deletes all particles.
    void clearParticles();
    
    // How many particles are currently in existance
    static int particleCount;
    
    std::list<Emitter*> emitters;
    
private:
    
    // The list of particles controled by this emitter.
    std::list<Particle*> particles;
};

#endif
