#ifndef _conf_included_conf_h
#define _conf_included_conf_h

#include <string>
#include <sstream>
#include <allegro.h>

// Handles software rendering.
extern class SoftwareRenderer {
public:
	
    // The screen BITMAP will be set to the current page or buffer
    // to be drawn to
	static BITMAP *screen;
    
	static bool ready;
    
    static const int raw           = 1;
    static const int double_buffer = 2;
    static const int page_flip     = 4;

private:
    BITMAP *buffer;
    BITMAP *pages[3];
    int page, pagecount;

    struct def {
#ifdef DEBUGMODE
        static const int mode        = GFX_AUTODETECT_WINDOWED;
#else
        static const int mode        = GFX_AUTODETECT_FULLSCREEN;
#endif
        static const int w           = 640;
        static const int h           = 480;
        static const int update_mode = raw | double_buffer | page_flip;
    };
    
    // These variables used to revert from failured modes
    int mode;
    int screen_w, screen_h;
    
    int set_gfx_mode(int,int,int);
    
    bool init_page_flipping();
    bool init_double_buffer();
    
    void post_draw_2_page();
    void post_draw_3_page();
    
    int update_mode;
    
public:
    // Disallows multiple instances
    SoftwareRenderer() throw(const char*);
    
    // Returns either 'raw', 'double_buffer' or 'page_flip' depending on
    // which of the rendering modes is being used.
    int get_render_mode();
    
    // Returns a string telling what mode is being used,
    const char *get_mode();
    
    // Starts up the software renderer.  mode should be either GFX_AUTODETECT_WINDOWED, or
    // GFX_AUTODETECT_FULLSCREEN.  w and h are the dimensions of the window
    // to create.  update_mode is all allowed modes OR'd together.
    //
    // For example to get direct screen access pass this as the update_mode:
    // raw
    //
    // To use page flipping and fail if it cannot be done on the current system,
    // pass this as the update_mode:
    // page_flip
    //
    // To use the best mode possible (trying page flipping first and going down),
    // pass this as the update_mode:
    // raw | double_buffer | page_flip
    //
    // Returns non-zero on failure
    int init(int mode = def::mode, int w = def::w, int h = def::h, int update_mode = def::update_mode);
    
    // Be sure to call this when you're done using the screen / window to
    // prevent memory leaks.
    void shutdown();
    
    // Should be called once you are done drawing the current frame to
    // SoftwareRenderer::screen.
    void post_draw();
} softwareRenderer;

#endif
