#ifndef STOPWATCH_H
#define STOPWATCH_H

// A class used for controling things over time.
// Able to start with a given number of milliseconds
// and continue ticking, giving the tick count
// untill the time the stopwatch was started with
// elapses.
//
// It is pauseable.
class stopwatch {
    
    unsigned int start_time; // The time that this stopwatch was started
    unsigned int pause_time; // the length this stopwatch should be active for
    
    bool ticking; // false once the time limit is over
    
    unsigned int timeOfPausing; // the time a pause of the stopwatch occured
    bool isPaused; // True if the stopwatch has been paused
    
public:
    
    // Pauses all stopwatches
    static void pauseAll();
    
    // Unpauses all stopwatches
    static void unpauseAll();
    
    stopwatch();
    ~stopwatch();
    
    // Begins the stopwatch with a given number of milliseconds.
    // time is the number of milliseconds.
    void start(unsigned int time);
    
    // Checks the status of the stopwatch.  Returns false if the stopwatch is
    // still ticking (ie its time hasnt elapsed yet.  Returns true when
    // the stopwatch's time has elapsed
    bool check();
    
    // Skips ahead to end the stopwatch's timer early
    void reset();
    
    // Pauses the stopwatch at its current tick.  Has no effect if already paused.
    void pause();
    
    // Resumes (unpauses) the stopwatch.  Has no effect if not already paused.
    void unpause();
    
    // The milliseconds that this stopwatch has been ticking for.
    // Starts at 0 and increments up to the value passed to start.
    unsigned int msecs();
};

#endif
