#ifndef THREAD_H
#define THREAD_H

#ifdef WIN32
#include <windows.h>
#else
#include <pthread.h>
#endif

// A controller and executer of a given thread
class Thread {
public:
    
    // proc is a thread entry point function
    typedef int (*Proc)(void*);
    
    // Starts a new thread with proc as the given entry point.
    // Will call proc with a given param.
    Thread(Proc proc, void *param = 0);
    
    // The dtor must not be called until the thread has finished executing
    ~Thread();
    
    // Returns true if the thread has finished executing
    bool done();
    
    // Returns the value returned by the proc function.
    int returnCode();
    
    // So ThreadProc can find the proc to call
    Proc proc;
    
    // So ThreadProc can pass param to the given proc
    void *param;
    
    int retCode;
    
    bool isDone;
    
private:
    
#ifdef WIN32
    
    HANDLE handle;
    
#else
    
    pthread_t pthread;
    pthread_attr_t attr;
    
#endif
};

#endif
