#ifndef WIDGET_H
#define WIDGET_H

#include "bitmap.h"
#include "eventreciever.h"
#include <list>
#include <string>
#include <map>

class Widget : public EventReciever {
public:
    
    Widget(int x_ = 0, int y_ = 0, int w_ = 0, int h_ = 0) : hasFocus(0), x(x_), y(y_), w(w_), h(h_),
        xcol(x), ycol(y), wcol(w), hcol(h)
    {
        attributes["rolloverCursor"] = "GUI/CURSOR/ROLLOVER/";
    }
    
    virtual ~Widget() { }
    
protected:
    
    // Capture events and send them to all the nessicary recievers.
    
    virtual Action sendKey(int key);
    virtual Action sendFocus();
    virtual Action sendUnfocus();
    virtual Action sendHover();
    virtual Action sendLeave();
    virtual Action sendClick();
    virtual Action sendRelease();
    virtual Action sendIdle();
    
    // Set to true when this widget has focus, false when it loses it
    bool hasFocus;
    
    // Called whenever the object needs to draw itself to the screen.
    // The first two parameters specify the upper lefthand corner of the screen and all widgets should draw
    // to x + xstart and y + ystart.
    //
    /* NAMES:           x,   y
     */
    virtual void draw(int, int, const Tint&) { }
    
    // Called after all attributes (x, y, w, h, attributes etc) are
    // loaded and set.
    virtual void load() { }
    
public:
    
    typedef std::map<std::string, std::string> Attributes;
    
    // Widget specific attributes.  For example a checkbox could have the key "checked" associated
    // with the value "1" or "0".
    Attributes attributes;
    
    // The x / y offsets from the upper left corner of the screen and
    // the width / height of the object.
    int x, y;
    int w, h;
    
    // The x / y offsets from the upper left corner of the screen and
    // the width / height of the collision box.  This box is used for
    // determining if the mouse is over a paticular widget.
    int xcol, ycol;
    int wcol, hcol;
    
    // The list of recievers of events for this widget.  The methods of the
    // EventRecieverAsync will be called after the widget's event method's.
    // You can either use the std::list methods to manipulate the recievers
    // or the + and += operators to do it.
    std::list<EventRecieverAsync*> eventRecievers;
    
    // GuiScreen handles all the widgets
    friend class GuiScreen;
};

// Operators that allow easy addition and subtraction of EventRecieverAsyncs
std::list<EventRecieverAsync*> operator +(EventRecieverAsync &eventReciever1, EventRecieverAsync &eventReciever2);
std::list<EventRecieverAsync*> operator +(const std::list<EventRecieverAsync*> &eventRecievers, EventRecieverAsync &eventReciever);
std::list<EventRecieverAsync*> &operator +=(std::list<EventRecieverAsync*> &eventRecievers1, const std::list<EventRecieverAsync*> &eventRecievers2);
std::list<EventRecieverAsync*> &operator +=(std::list<EventRecieverAsync*> &eventRecievers, EventRecieverAsync &eventReciever);

#endif
