#ifndef WIDGETS_BUTTON_H
#define WIDGETS_BUTTON_H

#include "widget.h"
#include "animation.h"
#include <string>

namespace widgets {
    
    class Button : public Widget {
    public:
        
        Button(int x = 0, int y = 0, int w = 0, int h = 0);
        
        virtual ~Button() {}
        
        // attributes can contain the following keys:
        //  "text", The text that is displayed over the button.
        //  "releaseSound", Name of sound to be played when this button is released.  An
        //   empty string means nothing will be played.
        
        // Called when the widget needs to be drawn.  xstart and ystart are added to the widget's
        // x / y position to deduce the drawing position.
        virtual void draw(int xstart, int ystart, const Tint &tint);
        
        // The events this widget needs to recieve
        virtual Action onHover();
        virtual Action onLeave();
        virtual Action onClick();
        virtual Action onRelease();
        
    private:
        
        // Returns true if this button is disabled
        bool isDisabled();
        
        animation inactiveLeft;
        animation inactiveCenter;
        animation inactiveRight;
        
        animation activeLeft;
        animation activeCenter;
        animation activeRight;
        
        animation hoverLeft;
        animation hoverCenter;
        animation hoverRight;
        
        animation *curLeft;
        animation *curCenter;
        animation *curRight;
    };
}

#endif
