#include "servhandler.h"
#include "world.h"
#include "log.h"
#include <vector>
#include <map>
using namespace std;
using namespace net;

extern map<Socket::Sock, unsigned int> ids;
extern vector<Socket> clients;

static void idle(char*, Socket)
{
    
}

void updateBuilding(unsigned int idb, const Building &building, Socket socket)
{
        char msg[25];
        
        *msg = 5;
        
        unsigned int id = htonl(idb);
        unsigned int type = htonl(building.type);
        int x = htonl(building.x);
        int y = htonl(building.y);
        unsigned int player = htonl(building.player);
        unsigned int track = htonl(building.track);
        
        memcpy(msg + 1, (char*)&id, 4);
        memcpy(msg + 5, (char*)&type, 4);
        memcpy(msg + 9, (char*)&x, 4);
        memcpy(msg + 13, (char*)&y, 4);
        memcpy(msg + 17, (char*)&player, 4);
        memcpy(msg + 21, (char*)&track, 4);
    
        socket.send(msg, sizeof(msg));
}

static void login(char *data, Socket socket)
{
    char response[2];
    
    response[0] = 1; // login response
    
    string username(data, 32);
    string password(data + 32, 32);
    
    username = username.c_str();
    password = password.c_str();
    
    map<unsigned int, User>::iterator usr;
    
    for(usr = world.users.begin(); usr != world.users.end(); ++usr)
        if(usr->second.checkUsername(username))
            break;
    
    if(usr == world.users.end()) {
        
        response[1] = 0; // reject
    }
    else if(!usr->second.checkPass(password.c_str())) {
        
        response[1] = 0; // reject
    }
    else {
        
        response[1] = 1; // accepted
        ids[socket.sock] = usr->first;
    }
    
    if(!socket.send(response, sizeof(response)))
        out << "BAD\n";
    
    map<unsigned int, Building>::iterator itr;
    
    for(itr = world.buildings.begin(); itr != world.buildings.end(); ++itr)
        updateBuilding(itr->first, itr->second, socket);
}

static void newuser(char *data, Socket socket)
{
    char response[5];
    
    response[0] = 2;
    
    string username(data, 32);
    string password(data + 32, 32);
    
    username = username.c_str();
    password = password.c_str();
    
    User user;
    
    user.x = 0;
    user.y = 0;
    user.setUsername(username);
    user.setPass(password);
    user.items[7] = 1500; // Give him some money
    
    int res = htonl(world.addUser(user));
    
    memcpy(response + 1, (char*)&res, 4);
    
    if(!socket.send(response, sizeof(response)))
        out << "BAD\n";
}

static void placetrack(char *data, Socket socket)
{
    char response[5];
    
    response[0] = 2;
    
    Track track;
    
    track.player = ids[socket.sock];
    track.x = ntohl(*(int*)&data[0]);
    track.y = ntohl(*(int*)&data[4]);
    track.directions = 0;
    track.buildings[0] = 0;
    track.buildings[1] = 0;
    
    int res = htonl(world.addTrack(track));
    
    memcpy(response + 1, (char*)&res, 4);
    
    if(!socket.send(response, sizeof(response)))
        out << "BAD\n";
}

static void buybuilding(char *data, Socket socket)
{
    char response[5];
    
    response[0] = 2;
    
    Building building;
    
    building.type = ntohl(*(unsigned int*)&data[0]);
    building.x = ntohl(*(int*)&data[4]);
    building.y = ntohl(*(int*)&data[8]);
    building.player = ids[socket.sock];
    building.track = 0;
        
    unsigned int id;
    
    int res = htonl(world.addBuilding(building, &id));
    
    id = htonl(id);
    
    out << "New building res: " << ntohl(res) << "\n";
    
    memcpy(response + 1, (char*)&res, 4);
    
    if(!socket.send(response, sizeof(response)))
        out << "BAD\n";
    
    if(res == 0) {
        
        char msg[25];
        
        *msg = 5;
        
        memcpy(msg + 1, (char*)&id, 4);
        memcpy(msg + 5, data, 12);
        
        unsigned int player = htonl(building.player);
        unsigned int track = htonl(building.track);
        
        memcpy(msg + 17, (char*)&player, 4);
        memcpy(msg + 21, (char*)&track, 4);
    
        for(unsigned int i = 0; i < clients.size(); ++i)
            clients[i].send(msg, sizeof(msg));
    }
}

static void removetrack(char *data, Socket socket)
{
    char response[5];
    
    response[0] = 2;
    
    int res = htonl(world.removeTrack(ntohl(*(unsigned int*)&data[0]), ids[socket.sock]));
    
    memcpy(response + 1, (char*)&res, 4);
    
    if(!socket.send(response, sizeof(response)))
        out << "BAD\n";
}

static void removebuilding(char *data, Socket socket)
{
    char response[5];
    
    response[0] = 2;
    
    unsigned int id = ntohl(*(unsigned int*)&data[0]);
    
    int res = htonl(world.removeBuilding(id, ids[socket.sock]));
    
    memcpy(response + 1, (char*)&res, 4);
    
    if(!socket.send(response, sizeof(response)))
        out << "BAD\n";
    
    if(ntohl(res) == 0) {
        
        char msg[5];
        
        *msg = 6;
        
        id = htonl(id);
        
        memcpy(msg + 1, (char*)&id, 4);
        
        for(unsigned int i = 0; i < clients.size(); ++i)
            clients[i].send(msg, sizeof(msg));
    }
}

static map<Socket::Sock, char> versions;

static void getversion(char *data, Socket socket)
{
    versions[socket.sock] = *data;
    
    out << "Version " << (int)*data << " conneted.\n";
}

int maxTag = 7;

unsigned int sizeOfData[] =
{
    0, // 0 - idle
    64, // 1 - login (32 username, 32 password)
    64, // 2 - new user (32 username, 32 password)
    8, // 3 - place track piece (4 x, 4 y, 4 direction)
    12, // 4 - buy building (4 typeid, 4 x, 4 y)
    4, // 5 - remove track
    4, // 6 - remove building
    1, // 7 - get version (1 version)
};

void (*tagHandlers[])(char*, Socket)  =
{
    idle, // 0
    login, // 1
    newuser, // 2
    placetrack, // 3
    buybuilding, // 4
    removetrack, // 5
    removebuilding, // 6
    getversion, // 7
};
