#include "datafile.h"
#include <cctype>
using namespace std;

DATAFILE *dat = 0;

datafile_t datafile;

datafile_t::datafile_t(const string &fname) : dat(0), filename(fname)
{
    
}

void datafile_t::set_filename(const string &fname)
{
    filename = fname;
}

string datafile_t::get_filename() const
{
    return filename;
}

bool datafile_t::load()
{
    return (dat = load_datafile(filename.c_str()));
}

void datafile_t::unload()
{
    if(dat)
        unload_datafile(dat);
    
    dat = 0;
    
    for(map<void*, CacheType>::iterator itr = loadfinds.begin();
        itr != loadfinds.end(); ++itr) {
        
        unload_datafile_object(itr->second.dat);
    }
    
    loadfinds.clear();
}

void *datafile_t::find(const std::string &objname, DATAFILE **datf)
{
    if(0 == dat)
        return 0;
    
    DATAFILE *obj = find_datafile_object(dat, objname.c_str());
    
    if(0 != datf)
        *datf = obj;
    
    if(0 == obj)
        return 0;
    
    return obj->dat;
}

namespace {
    
    string &tolower(string &str) {
        
        for(string::iterator itr = str.begin(); itr != str.end(); ++itr)
            *itr = ::tolower(*itr);
        
        return str;
    }
    
    string tolower(const string &str) {
        
        string ret(str);
        
        for(string::iterator itr = ret.begin(); itr != ret.end(); ++itr)
            *itr = ::tolower(*itr);
        
        return ret;
    }
}

void *datafile_t::loadfind(std::string objname, DATAFILE **datf)
{
    void *ret = 0;
    
    if((ret = find(objname, datf)))
        return ret;
    
    tolower(objname);
    
    DATAFILE *obj = 0;
    
    for(map<void*, CacheType>::iterator itr = loadfinds.begin(); itr != loadfinds.end(); ++itr) {
        
        if(objname == itr->second.objname) {
            
            obj = itr->second.dat;
        }
    }
    
    if(0 == obj) {
        
        obj = load_datafile_object(filename.c_str(), objname.c_str());
        
        if(obj) {
            
            CacheType ct;
            ct.dat = obj;
            ct.objname = objname;
            loadfinds.insert(make_pair(obj->dat, ct));
        }
        else
            return 0;
    }
    
    if(0 != datf)
        *datf = obj;
    
    return obj->dat;
}

void datafile_t::unloadfind(void *dat)
{
    if(dat) {
        
        map<void*, CacheType>::iterator loc = loadfinds.find(dat);
        
        if(loc != loadfinds.end()) {
            
            unload_datafile_object(loc->second.dat);
            loadfinds.erase(loc);
        }
    }
}

DATAFILE *datafile_t::get_datafile()
{
    return dat;
}
