#include "gui.h"
#include "log.h"
#include "conf.h"
#include "settings.h"
#include "sounds.h"
#include "timer.h"
#include <GL/glu.h>
#include <iostream>
using namespace std;

Gui::Gui(GuiState *s)
    : timerNeedsReset(0)
{
    guiState = s;
    
    bool quit = 0;
    int fpsUpdateCounter = 0;
    int fps = 0;
    int readings = 0;
    const unsigned int msecsPerTick = 20;
    Timer::time_t frame_tick = timer.usecs();
    Timer::time_t logic_tick = timer.msecs();
    Timer::time_t t;
    
    while(!quit) {
        
        s->draw();
        
        t = timer.usecs();
        
        if(!settings.averageFps) {
            
            fps = (int)(1000000 / (t - frame_tick));
            frame_tick = t;
        }
        else if(t != frame_tick) {
            
            readings++;
            fps = (fps * (readings - 1) + (int)(1000000 / (t - frame_tick))) / readings;
            frame_tick = t;
        }
        
        if(timerNeedsReset) {
            
            timerNeedsReset = 0;
            readings = 0;
            frame_tick = timer.usecs();
            logic_tick = timer.msecs();
        }
        
        while(timer.msecs() - logic_tick > msecsPerTick && !timerNeedsReset && !quit) {
            
            if(!settings.averageFps)
                s->fps(fps);
            else if(--fpsUpdateCounter < 0) {
                
                s->fps(fps);
                readings = 0;
                fps = 0;
                
                fpsUpdateCounter = 4;
            }
            
            logic_tick += msecsPerTick;
            
            if(s->think(*this) == GuiState::END)
                quit = 1;
            
            if(globalConfInstance->requestedQuitAsync()) {
                
                out << "Quitting from X close button click\n";
                quit = 1;
            }
        }
        
        GLenum errCode;
        
        if(globalConfInstance->getRenderingMode() == Conf::HARDWARE)
            if((errCode = glGetError()) != GL_NO_ERROR)
                err << "OpenGL Error: " << gluErrorString(errCode) << "\n";
        
        rest(1);
    }
}

void Gui::resetTimer()
{
    timerNeedsReset = 1;
}

int Gui::returnCode()
{
    return guiState->getReturnCode();
}

GuiState::~GuiState()
{
    
}
