#include "guiscreen.h"
#include "splashscreen.h"
#include "widgets_button.h"
#include "widgets_text.h"
#include "widgets_slider.h"
#include "widgets_checkbox.h"
#include "widgets_minibutton.h"
#include "widgets_listbox.h"
#include "widgets_input.h"
#include "widgets_image.h"
#include "readhash.h"
#include "settings.h"
#include "log.h"
#include <map>
#include <cctype>
#include <fstream>
using namespace std;
using namespace widgets;

GuiScreen *GuiScreen::active = 0;

GuiScreen::GuiScreen(const string &filename, datafile_t *datafile)
    : w(640), h(480), hasHover(0), hasFocus(0), oldClick(0),
      old_mouse_x(mouse_x), old_mouse_y(mouse_y), defPointer("GUI/CURSOR/"),
      curPointer(&defPointer), windowWidgetFound(0)
{
    xstart = 0;
    ystart = 0;
    
    if(datafile) {
        
        DATAFILE *datf;
        char *str = (char*)datafile->loadfind(filename, &datf);
        
        if(str) {
            istringstream instream(string(str, datf->size));
            
            out << "Loading GUI from datafile.\n";
            
            load(instream);
            
            return;
        }
    }
    
    ifstream instream(filename.c_str());
    
    out << "Loading GUI from file.\n";
    
    load(instream);
}

GuiScreen::~GuiScreen()
{
    for(list<Widget*>::iterator itr = widgets.begin(); itr != widgets.end(); ++itr)
        delete *itr;
    
    for(map<string, animation*>::iterator itr = cursors.begin(); itr != cursors.end(); ++itr)
        delete itr->second;
}

void GuiScreen::draw(const Tint &tint)
{
    for(list<Widget*>::reverse_iterator itr = widgets.rbegin(); itr != widgets.rend(); ++itr) {
        
        Tint temp;
        
        if((*itr)->attributes.count("r"))
            istringstream((*itr)->attributes["r"]) >> temp.r;
        
        if((*itr)->attributes.count("g"))
            istringstream((*itr)->attributes["g"]) >> temp.g;
        
        if((*itr)->attributes.count("b"))
            istringstream((*itr)->attributes["b"]) >> temp.b;
        
        if((*itr)->attributes.count("a"))
            istringstream((*itr)->attributes["a"]) >> temp.a;
        
        temp.r *= tint.r;
        temp.g *= tint.g;
        temp.b *= tint.b;
        temp.a *= tint.a;
        
        (*itr)->draw((*itr)->x, (*itr)->y, temp);
    }
    
    if(curPointer->get_bmp())
        curPointer->get_bmp()->blitToScreen(mouse_x, mouse_y, tint);
}

void GuiScreen::poll()
{
    bool newClick = mouse_b;
    int new_mouse_x = mouse_x;// * 640 / SCREEN_W - ((xstart - 100) + (640 - w) / 2);
    int new_mouse_y = mouse_y;// * 480 / SCREEN_H - ((ystart - 100) + (480 - h) / 2);
    
    active = this;
    
    // Send onClick message.
    if(newClick && !oldClick) {
        
        for(list<Widget*>::iterator itr = widgets.begin(); itr != widgets.end(); ++itr) {
            
            Widget &widget = **itr;
			
			//hasFocus = NULL;
            //widget.hasFocus = false;
			
            if(new_mouse_x > widget.xcol && new_mouse_x < widget.xcol + widget.wcol)
                if(new_mouse_y > widget.ycol && new_mouse_y < widget.ycol + widget.hcol) {
                    
                    if(hasFocus != &widget) {
                        
                        if(hasFocus)
                            hasFocus->hasFocus = false;
                        
                        hasFocus = &widget;
                        
                        widget.hasFocus = true;
                        
                        clear_keybuf();
                    }
                    
                    widget.sendClick();
                    
                    break;
                }
        }
    }
    
    // Send onRelease message.
    else if(!newClick && oldClick) {
        
        for(list<Widget*>::iterator itr = widgets.begin(); itr != widgets.end(); ++itr) {
            
            Widget &widget = **itr;
            
            if(new_mouse_x > widget.xcol && new_mouse_x < widget.xcol + widget.wcol)
                if(new_mouse_y > widget.ycol && new_mouse_y < widget.ycol + widget.hcol) {
                    
                    if(&widget == hasFocus)
                        widget.sendRelease();
                    
                    break;
                }
        }
    }
    
    // Poll hovering and send appropriate messages.
    if(new_mouse_x != old_mouse_x || new_mouse_y != old_mouse_y) {
        
        for(list<Widget*>::iterator itr = widgets.begin(); itr != widgets.end(); ++itr) {
            
            Widget &widget = **itr;
            
            if(new_mouse_x > widget.xcol && new_mouse_x < widget.xcol + widget.wcol)
                if(new_mouse_y > widget.ycol && new_mouse_y < widget.ycol + widget.hcol) {
                    
                    if(hasHover && &widget != hasHover) {
                        
                        hasHover->sendLeave();
                    }
                    
                    widget.sendHover();
                    hasHover = &widget;
                    
                    setCursor(widget.attributes["rolloverCursor"]);
                    
                    goto HoveringWidgetFound;
                }
        }
        
        // Hover has left this object and nothing is hovering anymore
        if(hasHover) {
            
            hasHover->sendLeave();
            hasHover = 0;
            
            curPointer = &defPointer;
        }
        
        HoveringWidgetFound:;
    }
    
    for(list<Widget*>::iterator itr = widgets.begin(); itr != widgets.end(); ++itr)
        (*itr)->sendIdle();
    
    oldClick = newClick;
    old_mouse_x = new_mouse_x;
    old_mouse_y = new_mouse_y;
}

bool GuiScreen::actionWaiting()
{
    int ignored;
    
    for(list<EventInfo>::iterator itr = eventRecievers.begin(); itr != eventRecievers.end(); ++itr) {
        
        if(itr->events.onKeyAsync(ignored) && !itr->keyAction.empty())
            generateAction(itr->keyAction, itr->widget);
        
        if(itr->events.onFocusAsync() && !itr->focusAction.empty())
            generateAction(itr->focusAction, itr->widget);
        
        if(itr->events.onUnfocusAsync() && !itr->unfocusAction.empty())
            generateAction(itr->unfocusAction, itr->widget);
        
        if(itr->events.onHoverAsync() && !itr->hoverAction.empty())
            generateAction(itr->hoverAction, itr->widget);
        
        if(itr->events.onLeaveAsync() && !itr->leaveAction.empty())
            generateAction(itr->leaveAction, itr->widget);
        
        if(itr->events.onClickAsync() && !itr->clickAction.empty())
            generateAction(itr->clickAction, itr->widget);
        
        if(itr->events.onReleaseAsync() && !itr->releaseAction.empty())
            generateAction(itr->releaseAction, itr->widget);
    }
    
    return !actions.empty();
}

string GuiScreen::getAction(Widget **commander)
{
    string ret;
    
    if(!actions.empty()) {
        
        ret = actions.front().first;
        
        if(commander)
            *commander = actions.front().second;
        
        actions.pop();
    }
    
    return ret;
}

void GuiScreen::generateAction(const string &action, Widget *sender)
{
    actions.push(make_pair(action, sender));
}

bool GuiScreen::hasWindow()
{
    return windowWidgetFound;
}

Widget *GuiScreen::findByKey(const string &key, const string &value)
{
    Widget *ret = 0;
    list<Widget*>::iterator itr = widgets.begin();
    
    for( ; !ret && itr != widgets.end(); ++itr)
        if((*itr)->attributes.count(key))
            if((*itr)->attributes[key] == value)
                ret = *itr;
    
    return ret;
}

void GuiScreen::setCursor(const string &path)
{
    if(path.empty())
        curPointer = &defPointer;
    else if(cursors[path])
        curPointer = cursors[path];
    else
        curPointer = cursors[path] = new animation(path);
}

void GuiScreen::load(istream &in)
{
    Widget *widget;
    string name;
    string line;
    map<string, string> hash;
    
    active = this;
    
    while(in.good()) {
        
        widget = 0;
        
        line.clear();
        
        getline(in, line);
        
        unsigned int i = 0;
        
        while((i = line.find('\\', i)) != string::npos && i < line.size() - 2) {
            
            if(line.at(i + 1) == 'n') {
                
                line.at(i) = '\n';
                line.erase(i + 1, 1);
            }
            else if(line.at(i + 1) == 't') {
                
                line.at(i) = '\t';
                line.erase(i + 1, 1);
            }
            
            i++;
        }
        
        istringstream stream(line);
        
        if(!(stream >> name))
            continue;
        
        if(name == "Logic")
            widget = new Widget; // Logic is a magic widget that doesnt actually exist
        else if(name == "OnLoad")
            widget = new Widget;
        else if(name == "Button")
            widget = new Button;
        else if(name == "MiniButton")
            widget = new MiniButton;
        else if(name == "Text")
            widget = new Text;
        else if(name == "CheckBox")
            widget = new CheckBox;
        else if(name == "Slider")
            widget = new Slider;
        else if(name == "ListBox")
            widget = new ListBox;
        else if(name == "Input")
            widget = new widgets::Input;
        else if(name == "Image")
            widget = new Image;
        else if(name == "Window") {
            
            readHash(stream, hash);
            
            if(!hash["w"].empty()) {
                
                istringstream in(hash["w"]);
                in >> w;
            }
            
            if(!hash["h"].empty()) {
                
                istringstream in(hash["h"]);
                in >> h;
            }
            
            title = hash["title"];
            
            if(hash["box"] != "off")
                windowWidgetFound = true;
            
            widget = 0;
        }
        
        if(widget) {
            
            // Read in widget specific data
            
            readHash(stream, widget->attributes);
            
            // Replace magic keywords with their values
            
            for(map<string, string>::iterator itr = widget->attributes.begin(); itr != widget->attributes.end(); ++itr) {
                
                if(itr->second == "settings::mute") {
                    ostringstream ss;
                    ss << settings.mute;
                    itr->second = ss.str();
                }
                
                if(itr->second == "settings::sfxVolume") {
                    ostringstream ss;
                    ss << settings.sfxVolume;
                    itr->second = ss.str();
                }
                
                if(itr->second == "settings::musicVolume") {
                    ostringstream ss;
                    ss << settings.musicVolume;
                    itr->second = ss.str();
                }
                
                if(itr->second == "settings::screenWidth") {
                    ostringstream ss;
                    ss << settings.screenWidth;
                    itr->second = ss.str();
                }
                
                if(itr->second == "settings::screenHeight") {
                    ostringstream ss;
                    ss << settings.screenHeight;
                    itr->second = ss.str();
                }
                
                if(itr->second == "settings::colorDepth") {
                    ostringstream ss;
                    ss << settings.colorDepth;
                    itr->second = ss.str();
                }
                
                if(itr->second == "settings::fullscreen") {
                    ostringstream ss;
                    ss << settings.fullscreen;
                    itr->second = ss.str();
                }
                
                if(itr->second == "settings::fpsDisplay") {
                    ostringstream ss;
                    ss << settings.fpsDisplay;
                    itr->second = ss.str();
                }
                
                if(itr->second == "settings::averageFps") {
                    ostringstream ss;
                    ss << settings.averageFps;
                    itr->second = ss.str();
                }
                
                if(itr->second == "settings::particleEffects") {
                    ostringstream ss;
                    ss << settings.particleEffects;
                    itr->second = ss.str();
                }
            }
            
            // Now read those x, y, w, h values
            
            stream >> widget->x;
            stream >> widget->y;
            stream >> widget->w;
            stream >> widget->h;
            
            widget->xcol = widget->x;
            widget->ycol = widget->y;
            widget->wcol = widget->w;
            widget->hcol = widget->h;
            
            if(widget->attributes.count("xcol")) {
                
                istringstream in(widget->attributes["xcol"]);
                
                in >> widget->xcol;
            }
            if(widget->attributes.count("ycol")) {
                
                istringstream in(widget->attributes["ycol"]);
                
                in >> widget->ycol;
            }
            if(widget->attributes.count("wcol")) {
                
                istringstream in(widget->attributes["wcol"]);
                
                in >> widget->wcol;
            }
            if(widget->attributes.count("hcol")) {
                
                istringstream in(widget->attributes["hcol"]);
                
                in >> widget->hcol;
            }
            
            // Now read all the items after the x, y, w, h values
            // (aka the widget-unspecific action properties).
            
            if(!readHash(stream, hash).empty()) {
                
                eventRecievers.resize(eventRecievers.size() + 1);
                EventInfo &eventInfo = eventRecievers.back();
                
                widget->eventRecievers += eventInfo.events;
                eventInfo.widget = widget;
                
                eventInfo.keyAction = hash["keyAction"];
                eventInfo.focusAction = hash["focusAction"];
                eventInfo.unfocusAction = hash["unfocusAction"];
                eventInfo.hoverAction = hash["hoverAction"];
                eventInfo.leaveAction = hash["leaveAction"];
                eventInfo.clickAction = hash["clickAction"];
                eventInfo.releaseAction = hash["releaseAction"];
            }
			
            widgets.push_back(widget);
        }
        
        if(name == "OnLoad")
            generateAction(widget->attributes["activationAction"], widget);
    }
    
    for(list<Widget*>::iterator itr = widgets.begin(); itr != widgets.end(); ++itr)
        (*itr)->load();
}
