#include "print.h"
#include "log.h"
#include "conf.h"
#include "software.h"
#include "font.h"
using namespace std;

namespace {
    
    FontRenderer fontRenderer;
}

Print::Print(int x_, int y_, const Tint &t, Align a)
    : alignment(a), tint(t), x(x_), y(y_), startingX(x)
{
    
}

void Print::print(string message)
{
    unsigned int loc;
    
    do {
        
        loc = message.find('\n');
        
        fontRenderer.drawString(message.substr(0, loc), x, y, tint);
        x += fontRenderer.stringWidth(message.substr(0, loc));
        
        if(loc != string::npos) {
            
            x = startingX;
            y += fontRenderer.height();
            
            if(message.size() <= loc + 1)
                break;
            
            message = message.substr(loc + 1);
        }
        
    } while(loc != string::npos);
}

FontRenderer &Print::getFontRenderer()
{
    return fontRenderer;
}

void Print::init()
{
    fontRenderer.load();
}

void Print::shutdown()
{
    fontRenderer.unload();
}

void Print::startInit()
{
    fontRenderer.startLoad();
}

bool Print::pollInit()
{
    return fontRenderer.loadStep() == -1;
}
