#include "settings.h"
#include "conf.h"
#include "readhash.h"
#include "sounds.h"
#include "log.h"
#include <fstream>
#include <sstream>
using namespace std;

Settings settings;

Settings::Settings()
    : filename("settings"), mute(0), sfxVolume(1), musicVolume(1),
      screenWidth(640), screenHeight(480), colorDepth(32),
#ifdef DEBUGMODE
      fullscreen(false), fpsDisplay(true),
#else
      fullscreen(true), fpsDisplay(false),
#endif
      averageFps(1), particleEffects(true), server("wipproject.zapto.org"),
      port(1423), debugTest1(false),
      
      oldScreenWidth(0), oldScreenHeight(0), oldColorDepth(0), oldFullscreen(0)
{
    
}

Settings::~Settings()
{
    
}

void Settings::load()
{
    ifstream in(filename.c_str());
    map<string, string> data;
    
    readHash(in, data);
    
    {
        istringstream in(data["mute"]);
        in >> mute;
    }
    {
        istringstream in(data["sfxVolume"]);
        in >> sfxVolume;
    }
    {
        istringstream in(data["musicVolume"]);
        in >> musicVolume;
    }
    {
        istringstream in(data["screenWidth"]);
        in >> screenWidth;
    }
    {
        istringstream in(data["screenHeight"]);
        in >> screenHeight;
    }
    {
        istringstream in(data["colorDepth"]);
        in >> colorDepth;
    }
    {
        istringstream in(data["fullscreen"]);
        in >> fullscreen;
    }
    {
        istringstream in(data["fpsDisplay"]);
        in >> fpsDisplay;
    }
    {
        istringstream in(data["averageFps"]);
        in >> averageFps;
    }
    {
        server = data["server"];
    }
    {
        istringstream in(data["port"]);
        in >> port;
    }
    {
        istringstream in(data["debugTest1"]);
        in >> debugTest1;
    }
}

void Settings::save()
{
    ofstream out(filename.c_str());
    
    out << "mute=" << (mute ? 1 : 0) << '\n'
        << "sfxVolume=" << sfxVolume << '\n'
        << "musicVolume=" << musicVolume << '\n'
        << "screenWidth=" << screenWidth << '\n'
        << "screenHeight=" << screenHeight << '\n'
        << "colorDepth=" << colorDepth << '\n'
        << "fullscreen=" << (fullscreen ? 1 : 0) << '\n'
        << "fpsDisplay=" << (fpsDisplay ? 1 : 0) << '\n'
        << "averageFps=" << (averageFps ? 1 : 0) << '\n'
        << "particleEffects=" << (particleEffects ? 1 : 0) << '\n'
        << "server=" << server << '\n'
        << "port=" << port << '\n'
        << "debugTest1=" << (debugTest1 ? 1 : 0);
}

Settings &Settings::operator =(Settings &settings)
{
    mute = settings.mute;
    sfxVolume = settings.sfxVolume;
    musicVolume = settings.musicVolume;
    screenWidth = settings.screenWidth;
    screenHeight = settings.screenHeight;
    colorDepth = settings.colorDepth;
    fullscreen = settings.fullscreen;
    fpsDisplay = settings.fpsDisplay;
    averageFps = settings.averageFps;
    particleEffects = settings.particleEffects;
    server = settings.server;
    port = settings.port;
    debugTest1 = settings.debugTest1;
    oldScreenWidth = settings.oldScreenWidth;
    oldScreenHeight = settings.oldScreenHeight;
    oldColorDepth = settings.oldColorDepth;
    oldFullscreen = settings.oldFullscreen;
    
    return *this;
}

void Settings::apply()
{
    using namespace Sounds;
    
    /*if(screenWidth != oldScreenWidth || screenHeight != oldScreenHeight || colorDepth != oldColorDepth || oldFullscreen != fullscreen) {
        
        if(globalConfInstance->changeResolution(fullscreen ? Conf::FULLSCREEN : Conf::WINDOWED, screenWidth, screenHeight, colorDepth)) {
            
            oldScreenWidth = screenWidth;
            oldScreenHeight = screenHeight;
            oldColorDepth = colorDepth;
            oldFullscreen = fullscreen;
        }
        else {
            
            screenWidth = oldScreenWidth;
            screenHeight = oldScreenHeight;
            colorDepth = oldColorDepth;
            fullscreen = oldFullscreen;
            
            if(!globalConfInstance->changeResolution(fullscreen ? Conf::FULLSCREEN : Conf::WINDOWED, screenWidth, screenHeight, colorDepth))
                throw "Error reverting color depth to previous value.";
        }
    }*/
    
    if(mute) {
        
        setSfxVolume(0);
    }
    else {
        
        setSfxVolume(sfxVolume);
    }
}
