#include "thread.h"

#ifdef WIN32

namespace {
    
    DWORD WINAPI threadProc(LPVOID param)
    {
        Thread &thread = *(Thread*)param;
        
        thread.retCode = thread.proc(thread.param);
        
        thread.isDone = true;
        
        return 0;
    }
}

Thread::Thread(Proc pr, void *pa)
    : proc(pr), param(pa), retCode(0), isDone(0)
{
    handle = CreateThread(NULL, 0, threadProc, this, 0, NULL);
    
    if(handle == NULL)
        throw "Failure in CreateThread";
}

Thread::~Thread()
{
    CloseHandle(handle);
}

#else

namespace {
    
    void *threadProc(void *param)
    {
        Thread &thread = *(Thread*)param;
        
        thread.retCode = thread.proc(thread.param);
        
        thread.isDone = true;
        
        return 0;
    }
}

Thread::Thread(Proc pr, void *pa)
    : proc(pr), param(pa), retCode(0), isDone(0)
{
    pthread_attr_init(&attr);
    
    if(0 != pthread_create(&pthread, &attr, &threadProc, this))
        throw "Failure in pthread_create";
}

Thread::~Thread()
{
    pthread_attr_destroy(&attr);
}

#endif

bool Thread::done()
{
    return isDone;
}

int Thread::returnCode()
{
    return retCode;
}
