#include "widgets_input.h"
#include "widgets_button.h"
#include "guiscreen.h"
#include "print.h"
using namespace std;
using namespace widgets;

Input::Input()
    : xstart(0), ystart(0), cursorPos(0), scrollPos(0), 
      
      textLeft("GUI/INPUTBOX/ARROW/LEFT/"), textRight("GUI/INPUTBOX/ARROW/RIGHT/"),
      left("GUI/INPUTBOX/LEFT/"), center("GUI/INPUTBOX/CENTER/"), right("GUI/INPUTBOX/RIGHT/"),
      cursor("GUI/INPUTBOX/CARET/")
{
    w = 150;
    h = 32;
    
    attributes["rolloverCursor"] = "GUI/CURSOR/TEXT/";
}

void Input::draw(int xstart, int ystart, const Tint &tint)
{
    string &value = attributes["value"];
    
    int x = xstart;
    int y = ystart;
    
    this->xstart = xstart;
    this->ystart = ystart;
    
    if(left.get_bmp())
        left.get_bmp()->blitToScreen(x, y, tint);
    
    if(center.get_bmp())
        center.get_bmp()->specialBlitToScreen(x + 32, y, (w - 64.0f) / center.get_bmp()->getw(), 1, 0, tint);
    
    if(right.get_bmp())
        right.get_bmp()->blitToScreen(x + w - 32, y, tint);
    
    if(attributes["special"] == "password")
        Print(x + 16, y + 10, tint) << string(visibleChars(), '*');
    else
        Print(x + 16, y + 10, tint) << value.substr(scrollPos, visibleChars());
    
    if(cursor.get_bmp() && hasFocus)
        cursor.get_bmp()->blitToScreen(x + getCursorPixel() - cursor.get_bmp()->getw() / 2 + 16, y + 5, tint);
    
    if(scrollPos > 0 && textLeft.get_bmp())
        textLeft.get_bmp()->blitToScreen(x - 32, y + 8, tint);
    
    if(value.size() > (unsigned)(scrollPos + visibleChars()) && textRight.get_bmp())
        textRight.get_bmp()->blitToScreen(x + w, y + 8, tint);
}

Input::Action Input::onHover()
{
    if(mouse_b)
        setCursorPixel(mouse_x - xstart - x);
    
    return OK;
}

Input::Action Input::onClick()
{
    if(mouse_b)
        setCursorPixel(mouse_x - xstart - x);
    
    return OK;
}

Input::Action Input::onIdle()
{
    string &value = attributes["value"];
    
    if(hasFocus && keypressed()) {
        
        int val = readkey();
        
        if((val >> 8) == KEY_ENTER) {
            
            if(attributes.count("enterAction"))
                GuiScreen::active->generateAction(attributes["enterAction"], this);
        }
        else if((val >> 8) == KEY_BACKSPACE) {
            
            if(cursorPos != 0 && !value.empty()) {
                
                value.erase(cursorPos - 1, 1);
                
                setCursorPos(cursorPos - 1);
                boundsCheckScroll();
            }
        }
        else if((val >> 8) == KEY_DEL) {
            
            if(cursorPos < (signed)value.size() && !value.empty()) {
                
                value.erase(cursorPos, 1);
                
                setCursorPos(cursorPos);
                boundsCheckScroll();
            }
        }
        else if((val >> 8) == KEY_LEFT) {
            
            setCursorPos(cursorPos - 1);
        }
        else if((val >> 8) == KEY_RIGHT) {
            
            setCursorPos(cursorPos + 1);
        }
        else if((val >> 8) == KEY_HOME) {
            
            setCursorPos(0);
        }
        else if((val >> 8) == KEY_END) {
            
            setCursorPos(value.size());
        }
        else if(val & 0xff) {
            
            if(!attributes.count("restrictTo") || attributes["restrictTo"].find((char)(val & 0xff)) != string::npos) {
                
                value.insert(cursorPos, 1, val & 0xff);
                setCursorPos(cursorPos + 1);
                
                if(attributes.count("checkOnType")) {
                    
                    Widget *widget = GuiScreen::active->findByKey("name", attributes["checkOnType"]);
                    
                    if(widget)
                        widget->attributes["checked"] = "1";
                }
                
                if(attributes.count("uncheckOnType")) {
                    
                    Widget *widget = GuiScreen::active->findByKey("name", attributes["uncheckOnType"]);
                    
                    if(widget)
                        widget->attributes["checked"] = "0";
                }
            }
        }
        
        if(value.empty()) {
            
            if(attributes.count("checkOnEmpty")) {
                
                Widget *widget = GuiScreen::active->findByKey("name", attributes["checkOnEmpty"]);
                
                if(widget)
                    widget->attributes["checked"] = "1";
            }
            
            if(attributes.count("uncheckOnEmpty")) {
                
                Widget *widget = GuiScreen::active->findByKey("name", attributes["uncheckOnEmpty"]);
                
                if(widget)
                    widget->attributes["checked"] = "0";
            }
        }
        
        if(attributes.count("syncValueWith"))
            syncValue();
    }
    
    return OK;
}

void Input::load()
{
    if(attributes.count("syncValueWith"))
        syncValue();
    
    setCursorPos(attributes["value"].size());
}

void Input::syncValue()
{
    string &value = attributes["value"];
    
    Widget *widget = GuiScreen::active->findByKey("name", attributes["syncValueWith"]);
    
    if(widget)
        if(attributes.count("isConfirm"))
            widget->attributes["confirm"] = value;
        else
            widget->attributes["value"] = value;
}

int Input::visibleChars()
{
    string &value = attributes["value"];
    
    int w = this->w - 32;
    
    for(unsigned int i = scrollPos; i < value.size(); i++) {
        
        w -= getWidth(value.at(i));
        
        if(w <= 0)
            return i - scrollPos;
    }
    
    return value.size() - scrollPos;
}

int Input::getWidth(string str)
{
    FontRenderer &font = Print::getFontRenderer();
    
    if(attributes["special"] == "password")
        str = string(str.size(), '*');
    
    return font.stringWidth(str);
}

int Input::getWidth(char c)
{
    FontRenderer &font = Print::getFontRenderer();
    
    if(attributes["special"] == "password")
        return font.width('*');
    else
        return font.width(c);
}

int Input::getCursorPixel()
{
    string &value = attributes["value"];
    
    int ret = 0;
    
    for(int i = scrollPos; i < cursorPos; i++)
        ret += getWidth(value.at(i));
    
    return ret;
}

void Input::setCursorPixel(int x)
{
    string &value = attributes["value"];
    
    int newPos, tmp;
    string::iterator itr = value.begin();
    
    //x -= 16;
    
    for(newPos = 0; newPos < scrollPos && itr != value.end(); newPos++)
        ++itr;
    
    for(; itr != value.end(); ++itr, ++newPos) {
        
        tmp = getWidth(*itr);
        
        if(x <= tmp / 2)
            break;
        
        x -= tmp;
    }
    
    setCursorPos(newPos);
}

void Input::setCursorPos(int newPos)
{
    string &value = attributes["value"];
    
    if(newPos > (signed)value.size())
        newPos = value.size();
    
    if(newPos < 0)
        newPos = 0;
    
    cursorPos = newPos;
    
    boundsCheckScroll();
}

void Input::boundsCheckScroll()
{
    string &value = attributes["value"];
    
    if(scrollPos >= (signed)value.size())
        scrollPos = value.size() - 1;
    if(scrollPos < 0)
        scrollPos = 0;
    
    if(cursorPos < scrollPos)
        scrollPos = cursorPos;
    else while(cursorPos > scrollPos && getWidth(value.substr(scrollPos, cursorPos - scrollPos)) > w - 32)
        scrollPos++;
}
