#include "widgets_listbox.h"
#include "print.h"
#include "log.h"
using namespace std;
using namespace widgets;

#define elementspacing 11

ListBox *ListBox::active = 0;

ListBox::ListBox()
    : hasHover(0), scroll(0), xstart(0), ystart(0),
      mouseWheelPos(mouse_z), hasMouse(0), 
      
      scrollbar("gui/scrollbar/"),
      scrollpoint("gui/scrollpoint/"),
      
      buttonActive("gui/minibutton/active/"),
      buttonInactive("gui/minibutton/inactive/"),
      buttonHover("gui/minibutton/hover/"),
      
      up(&buttonInactive), down(&buttonInactive)
{
}

ListBox::Element *ListBox::getLower(Element *e)
{
    for(vector<Element*>::iterator itr = elements.begin(); itr != elements.end(); ++itr)
        if(itr != elements.begin() && *itr == e)
            return *(--itr);
    
    return e;
}

ListBox::Element *ListBox::getHigher(Element *e)
{
    for(vector<Element*>::iterator itr = elements.begin(); itr != elements.end(); ++itr)
        if(itr + 1 != elements.end() && *itr == e)
            return *(++itr);
    
    return e;
}

void ListBox::scrollTo(Element *e)
{
    int y = 0;
    
    for(vector<Element*>::iterator itr = elements.begin(); itr != elements.end(); ++itr) {
        
        if(*itr == e) {
            
            scroll = MAX(y - h + (*itr)->h, scroll);
            scroll = MIN(y, scroll);
        }
        
        y += (*itr)->h + elementspacing;
    }
}

ListBox::Action ListBox::onHover()
{
    active = this; 
    
    up = overUp() ? &buttonHover : &buttonInactive;
    down = overDown() ? &buttonHover : &buttonInactive;
    
    if(!hasMouse) {
        
        mouseWheelPos = mouse_z;
        hasMouse = true;
    }
    
    if(overScrollbar()) {
        
        if(mouse_b && !overUp() && !overDown())
            setScroll();
    }
    else if(!overUp() && !overDown()) {
        
        int y = this->y + ystart - scroll;
        
        for(unsigned int i = 0; i < elements.size(); i++) {
            
            y += elements[i]->h + elementspacing;
            
            if(mouse_y < y) {
                
                if(hasHover)
                    hasHover->onLeave();
                
                hasHover = elements[i];
                
                return elements[i]->onHover(mouse_x - (x + xstart), mouse_y - y + elements[i]->h);
            }
        }
    }
    
    if(hasHover) {
        
        hasHover->onLeave();
        hasHover = 0;
    }
    
    return OK;
}

ListBox::Action ListBox::onLeave()
{
    active = this; 
    
    up = &buttonInactive;
    down = &buttonInactive;
    
    hasMouse = false;
    
    if(hasHover)
        hasHover->onLeave();
    
    hasHover = 0;
    
    return OK;
}

ListBox::Action ListBox::onClick()
{
    active = this; 
    
    if(overUp())
        up = &buttonActive;
    else if(overDown())
        down = &buttonActive;
    else if(overScrollbar()) {
        
        hasMouse = true;
        setScroll();
    }
    
    if(!overScrollbar() && !overUp() && !overDown()) {
        
        int y = this->y + ystart - scroll;
        
        for(unsigned int i = 0; i < elements.size(); i++) {
            
            y += elements[i]->h + elementspacing;
            
            if(mouse_y < y) {
                
                return elements[i]->onClick(mouse_x - (x + xstart), mouse_y - y + elements[i]->h);
            }
        }
    }
    
    return OK;
}

ListBox::Action ListBox::onRelease()
{
    active = this; 
    
    if(overUp())
        scroll -= h / 3;
    else if(overDown())
        scroll += h / 3;
    
    wrapScroll();
    
    up = overUp() ? &buttonHover : &buttonInactive;
    down = overDown() ? &buttonHover : &buttonInactive;
    
    hasMouse = true;
    
    if(mouse_x - xstart < x + w - 28) {
        
        int y = this->y + ystart - scroll;
        
        for(unsigned int i = 0; i < elements.size(); i++) {
            
            y += elements[i]->h + elementspacing;
            
            if(mouse_y < y) {
                
                return elements[i]->onRelease(mouse_x - (x + xstart), mouse_y - y + elements[i]->h);
            }
        }
    }
    
    return OK;
}

ListBox::Action ListBox::onIdle()
{
    ListBox::Action ret = OK;
    
    active = this; 
    
    if(hasMouse) {
        
        scroll += (mouseWheelPos - mouse_z) * 50;
        mouseWheelPos = mouse_z;
    }
    
    wrapScroll();
    
    for(unsigned int i = 0; i < elements.size() && ret == OK; i++)
        ret = elements[i]->onIdle();
    
    return ret;
}

void ListBox::draw(int xstart, int ystart, const Tint &tint)
{
    this->xstart = xstart;
    this->ystart = ystart;
    
    int x = this->x;
    int y = this->y;
    
    int sy, eh;
    
    // Loop through elements drawing at proper coords and offsets
    
    for(unsigned int i = 0; i < elements.size(); i++) {
        
        if(y - scroll + elements[i]->h + elementspacing * (signed)i > this->y) {
            
            sy = MAX(0, this->y - (y - scroll));
            eh = MIN(elements[i]->h, scroll + h - y);
            
            Bitmap::scissorStart(this->x + xstart - 4, this->y + ystart, w - 30, h);
            
            elements[i]->draw(0, sy, x + xstart, y - scroll + ystart + sy + (elementspacing * i), w - 32, eh, tint);
            
            Bitmap::scissorEnd();
        }
        
        y += elements[i]->h;
        
        if(y - scroll > h)
            break;
    }
    
    // Draw scroll bar with buttons
    
    x = this->x + xstart;
	y = this->y + ystart - 2;
    
    if(up->get_bmp())
        up->get_bmp()->blitToScreen(x + (w - 30), y, tint);
    
    Print::getFontRenderer().draw('<', x + (w - 20), y + 3, tint);
    
    if(scrollbar.get_bmp())
        scrollbar.get_bmp()->specialBlitToScreen(x + (w - 22), y + 30, 1, float(h - 56) / scrollbar.get_bmp()->geth(), 0, tint);
    
    if(scrollpoint.get_bmp())
        scrollpoint.get_bmp()->blitToScreen(x + (w - 30), y + 22 + (getInsideHeight() ? (h - 73) * scroll / getInsideHeight() : 0), tint);
    
    if(down->get_bmp())
        down->get_bmp()->blitToScreen(x + (w - 30), y + (h - 28), tint);
    
    Print::getFontRenderer().draw('>', x + (w - 20), y + (h - 23), tint);
}

bool ListBox::overUp()
{
    const int mx = mouse_x - xstart;
    const int my = mouse_y - ystart;
    const int y = this->y - 2;
    
    return mx >= x + (w - 24) && mx <= x + w + 8 && my >= y && my <= y + 26;
}

bool ListBox::overDown()
{
    const int mx = mouse_x - xstart;
    const int my = mouse_y - ystart;
    const int y = (this->y + h) - 30;
    
    return mx >= x + (w - 24) && mx <= x + w + 8 && my >= y && my <= y + 26;
}

bool ListBox::overScrollbar()
{
    const int mx = mouse_x - xstart;
    const int my = mouse_y - ystart;
    
    return mx >= x + (w - 28) && mx <= x + w && my >= y + 27 && my <= y + (h - 45);
}

void ListBox::setScroll()
{
    const int my = (mouse_y - ystart) - 7;
    const int mrange = my - (y + 25);
    const int range = (y + (h - 53)) - (y + 35);
    
    scroll = getInsideHeight() * (mrange) / range;
    
    wrapScroll();
}

void ListBox::wrapScroll()
{
    if(scroll < 0)
        scroll = 0;
    else if(scroll > getInsideHeight())
        scroll = getInsideHeight();
}

int ListBox::getInsideHeight()
{
    int ret = -h;
    
    for(vector<Element*>::iterator loc = elements.begin(); loc != elements.end(); ++loc)
        ret += (*loc)->h + elementspacing;
	
    ret-=elementspacing;
	
    if(ret < 0)
        ret = 0;
    
    return ret;
}
