#include "widgets_slider.h"
#include "log.h"
#include "print.h"
#include <sstream>
using namespace std;
using namespace widgets;

namespace {
    
    // Padding inbetween buttons and centerpice
    const int pad = 3;
}

Slider::Slider()
    : xstart(0), ystart(0), hasClicked(0),
      
      left("GUI/SLIDER/LEFT/"),
      right("GUI/SLIDER/RIGHT/"),
      center("GUI/SLIDER/CENTER/"),
      meter("GUI/SLIDER/METER/"),
        
      minibuttonActive("GUI/MINIBUTTON/ACTIVE/"),
      minibuttonInactive("GUI/MINIBUTTON/INACTIVE/"),
      minibuttonHover("GUI/MINIBUTTON/HOVER/"),
      
      leftMinibutton(&minibuttonInactive),
      rightMinibutton(&minibuttonInactive)
{
    
}

void Slider::draw(int xstart, int ystart, const Tint &tint)
{
    int mFontWidth = Print::getFontRenderer().width('-');
    int pFontWidth = Print::getFontRenderer().width('+');
    int fontHeight = Print::getFontRenderer().height();
    
    this->xstart = xstart;
    this->ystart = ystart;
    
    int x = this->x + xstart;
    int y = this->y + ystart;
    
    if(leftMinibutton->get_bmp())
        leftMinibutton->get_bmp()->blitToScreen(x, y, tint);
    
    x += 16 - mFontWidth / 2;
    
    Print(x, y + (16 - fontHeight / 2) + 1, tint) << '-';
    
    x = this->x + xstart + 32 + pad;
    
    if(left.get_bmp())
        left.get_bmp()->blitToScreen(x, y, tint);
    
    x += 16;
    
    if(center.get_bmp())
        center.get_bmp()->specialBlitToScreen(x, y, (w - (96 + 2 * pad)) / 16, 1, 0, tint);
    
    istringstream in(attributes["value"]);
    in >> f;
    
    if(meter.get_bmp())
        meter.get_bmp()->specialBlitToScreen(x, y + 5, (w - (96 + 2 * pad)) / 16 * f, 1.4, 0, tint);
    
    x += ((w - (96 + 2 * pad)) / 16) * 16;
    
    if(right.get_bmp())
        right.get_bmp()->blitToScreen(x, y, tint);
    
    x += 16 + pad;
    
    if(rightMinibutton->get_bmp())
        rightMinibutton->get_bmp()->blitToScreen(x, y, tint);
    
    x += 16 - pFontWidth / 2;
    
    Print(x, y + (16 - fontHeight / 2) + 1, tint) << '+';
}

Slider::Action Slider::onHover()
{
    if(overDecrement() && leftMinibutton != &minibuttonActive)
        leftMinibutton = &minibuttonHover;
    else
        leftMinibutton = &minibuttonInactive;
    
    if(overIncrement() && leftMinibutton != &minibuttonActive)
        rightMinibutton = &minibuttonHover;
    else
        rightMinibutton = &minibuttonInactive;
    
    if(mouse_b & 1) {
        
        overCenter(&f);
        
        ostringstream ss;
        ss << f;
        attributes["value"] = ss.str();
    }
    
    return OK;
}

Slider::Action Slider::onLeave()
{
    hasClicked = false;
    
    leftMinibutton = &minibuttonInactive;
    rightMinibutton = &minibuttonInactive;
    
    return OK;
}

Slider::Action Slider::onClick()
{
    hasClicked = true;
    
    if(overDecrement())
        leftMinibutton = &minibuttonActive;
    else if(overIncrement())
        rightMinibutton = &minibuttonActive;
    else {
        
        overCenter(&f);
        
        ostringstream ss;
        ss << f;
        attributes["value"] = ss.str();
    }
    
    return OK;
}

Slider::Action Slider::onRelease()
{
    hasClicked = false;
    
    istringstream in(attributes["value"]);
    in >> f;
    
    if(overIncrement())
        f += 0.125;
    else if(overDecrement())
        f -= 0.125;
    
    if(f > 1)
        f = 1;
    
    if(f < 0)
        f = 0;
    
    ostringstream ss;
    ss << f;
    attributes["value"] = ss.str();
    
    if(overDecrement())
        leftMinibutton = &minibuttonHover;
    else
        leftMinibutton = &minibuttonInactive;
    
    if(overIncrement())
        rightMinibutton = &minibuttonHover;
    else
        rightMinibutton = &minibuttonInactive;
    
    return OK;
}

bool Slider::overDecrement()
{
    int mx = mouse_x - xstart;
    int my = mouse_y - ystart;
    
    return mx >= x && mx <= x + 32 && my >= y && my <= y + 32;
}

bool Slider::overIncrement()
{
    int mx = mouse_x - xstart;
    int my = mouse_y - ystart;
    int x = (this->x + w) - 32;
    
    return mx >= x && mx <= x + 32 && my >= y && my <= y + 32;
}

bool Slider::overCenter(float *where)
{
    int mx = mouse_x - xstart - 12;
    int my = mouse_y - ystart;
    int x = this->x + 32 + pad;
    int w = ((this->w - (96 + 2 * pad)) / 16) * 16;
    
    if(mx >= x && mx <= x + w && my >= y && my <= y + 32) {
        
        if(where)
            *where = (float)(mx - x) / w;
        
        return true;
    }
    
    return false;
}
