#include "world.h"
#include "log.h"
#include <fstream>
#include <sstream>
using namespace std;

World world;

namespace {
    
    // Finds the next available Id for a given map
    template<class T1, class T2>
    T1 nextId(const map<T1, T2> &t, T1 start = 1)
    {
        while(t.count(start))
        start++;
        
        return start;
    }
}

int World::takeResources(unsigned int id, unsigned int player, int *amount)
{
    map<unsigned int, Resource>::iterator res = resources.find(id);
    
    if(res == resources.end())
        return 4;
    
    map<unsigned int, User>::iterator usr = users.find(player);
    
    if(usr == users.end())
        return 3;
    
    if(!res->second.building || !buildings.count(res->second.building) || buildings[res->second.building].player != player)
        return 2;
        
    *amount = res->second.takeResource(*amount);
    
    if(*amount == 0)
        return 1;
    
    return 0;
}

int World::addBuilding(const Building &building, unsigned int *id)
{
    if(!buildingDb.buildings.count(building.type))
        return 18;
    
    map<unsigned int, Track>::iterator trk = tracks.find(building.track);
    
    //if(trk == tracks.end())
    //    return 15;
    
    map<unsigned int, User>::iterator usr = users.find(building.player);
    
    //if(usr == users.end())
    //    return 3;
    
    //if(usr->second.items[7] < buildingDb.buildings[building.type].cost)
    //    return 7;
    
    //if(building.x < trk->second.x - 1) return 5;
    //if(building.x > trk->second.x + 1) return 5;
    //if(building.y < trk->second.y - 1) return 5;
    //if(building.y > trk->second.y + 1) return 5;
    
    map<unsigned int, Resource>::iterator res;
    
    for(res = resources.begin(); res != resources.end(); ++res) {
        
        if(building.x < res->second.x - res->second.radius) continue;
        if(building.x > res->second.x + res->second.radius) continue;
        if(building.y < res->second.y - res->second.radius) continue;
        if(building.y > res->second.y + res->second.radius) continue;
        
        break;
    }
    
    //if(res == resources.end() || res->second.building)
    //    return 8;
    
    //if(trk->second.player != building.player)
    //    return 6;
    
    usr->second.items[7] -= buildingDb.buildings[building.type].cost;
    
    res->second.building = nextId(buildings);
    
    if(id)
        *id = res->second.building;
    
    buildings[res->second.building] = building;
    
    return 0;
}

int World::removeBuilding(unsigned int id, unsigned int player)
{
    map<unsigned int, Building>::iterator bld = buildings.find(id);
    
    if(bld == buildings.end())
        return 10;
    
    map<unsigned int, User>::iterator usr = users.find(player);
    
    if(usr == users.end())
        return 3;
    
    if(bld->second.player != player)
        return 9;
    
    if(!buildingDb.buildings.count(bld->second.type))
        return 18;
    
    //if(!bld->second.track || !tracks.count(bld->second.track))
    //    return 20;
    
    //Track &t = tracks[bld->second.track];
    
    //if(t.buildings[0] == bld->second.track)
    //    t.buildings[0] = 0;
    //else if(t.buildings[2] == bld->second.track)
    //    t.buildings[1] = 0;
    //else
     //   return 21;
    
    usr->second.items[7] += buildingDb.buildings[bld->second.type].cost / 2;
    
    buildings.erase(bld);
    
    return 0;
}

int World::addUser(User user)
{
    user.setUsername(user.username); // make username lowercase
    
    for(string::iterator itr = user.username.begin(); itr != user.username.end(); ++itr) {
        
        if(!isalpha(*itr) && !isdigit(*itr) && *itr != '_')
            return 19;
    }
    
    map<unsigned int, User>::iterator itr;
    
    for(itr = users.begin(); itr != users.end(); ++itr)
        if(itr->second.username == user.username)
            return 11;
    
    users[nextId(users)] = user;
    
    return 0;
}

int World::addTerminal(const Terminal &terminal)
{
    map<unsigned int, User>::iterator user = users.find(terminal.owner_id);
    if(user == users.end())
        return 3;
    if(user->second.items[7] < 0)
        return 7;
    if(!cities.count(terminal.city))
        return 17;
    if(cities[terminal.city].terminals.size() > 8)
        return 12;
        
    terminals[nextId(terminals)] = terminal;
    
    return 0;
}

int World::removeTerminal(unsigned int id, unsigned int player)
{
    map<unsigned int, Terminal>::iterator terminal = terminals.find(player);
    if(terminal == terminals.end())
        return 12;
    map<unsigned int, User>::iterator user = users.find(player);
    if(user == users.end())
        return 3;
    if(user->first != terminal->second.owner_id)
        return 9;
    
    terminals.erase(terminal);
    
    return 0;
}

int World::addTrack(const Track &track)
{
    map<unsigned int, User>::iterator usr = users.find(track.player);
    
//     if(usr == users.end())
//         return 3;
//     
//     if(usr->second.items[7] < itemDb.items[1].cost)
//         return 7;
    
    map<unsigned int, Track>::iterator trk;
    
    for(trk = tracks.begin(); trk != tracks.end(); ++trk) {
        
        if(track.x == trk->second.x && track.y == trk->second.y)
            return 14;
    }
    
    map<unsigned int, Building>::iterator bld;
    
    for(bld = buildings.begin(); bld != buildings.end(); ++bld) {
        
        if(track.x == bld->second.x && track.y == bld->second.y)
            return 14;
    }
    
//     usr->second.items[7] -= itemDb.items[1].cost;
    
    unsigned int id = nextId(tracks);
    tracks[id] = track;
    
    std::vector<unsigned int> adj;
    tracks[id].find_adjacent(adj);
    
    return 0;
}

int World::removeTrack(unsigned int id, unsigned int player)
{
    map<unsigned int, Track>::iterator trk = tracks.find(id);
    
    if(trk == tracks.end())
        return 15;
    
    map<unsigned int, User>::iterator usr = users.find(player);
    
    if(usr == users.end())
        return 3;
    
    if(trk->second.player != player)
        return 9;
    
    tracks.erase(trk);
    
    return 0;
}

void World::loadCities(const string &filename)
{
    cities.clear();
    
    string line;
    
    ifstream in(filename.c_str());
    
    getline(in, line);
    
    if(line.find("cities") != 0)
        throw "Invalid cities file";
    
    unsigned int i = line.find("v");
    
    if(i == string::npos)
        throw "Unable to find version number in cities file";
    
    int ver = 0;
    
    istringstream verss(line.substr(i));
    
    verss.ignore();
    
    verss >> ver;
    
    if(ver != 1)
        throw "Invalid version number for cities file";
    
    while(in.good()) {
        
        getline(in, line);
        
        istringstream data(line);
        
        unsigned int id = 0;
        
        data >> id;
        
        if(id) {
            
            City &city = cities[id];
            
            // Read in all data (data is only for this one line)
            data >> city.name;
            data >> city.x;
            data >> city.y;
            data >> city.terminals[0];
            data >> city.terminals[1];
            data >> city.terminals[2];
            data >> city.terminals[3];
            data >> city.terminals[4];
            data >> city.terminals[5];
            data >> city.terminals[6];
        }
    }
    
    out << "Loaded cities, cities: " << cities.size() << "\n";
}

void World::saveCities(const string &filename) const
{
    ofstream file(filename.c_str());
    
    file << "cities v1\n";
    
    std::map<unsigned int, City>::const_iterator itr;
    
    for(itr = cities.begin(); itr != cities.end(); ++itr)
    {
        file << itr->first << " "
             << itr->second.name << " "
             << itr->second.x << " "
             << itr->second.y;
        map<int, unsigned int>::const_iterator j;
#       define SAVE_T(id) j = itr->second.terminals.find(id); \
            if(j == itr->second.terminals.end()) file << " 0"; \
            else file << " " << j->second;
        SAVE_T(0)
        SAVE_T(1)
        SAVE_T(2)
        SAVE_T(3)
        SAVE_T(4)
        SAVE_T(5)
        SAVE_T(6)
#       undef SAVE_T
        file << "\n";
    }
}

void World::loadResources(const string &filename)
{
    resources.clear();
    
    string line;
    
    ifstream in(filename.c_str());
    
    getline(in, line);
    
    if(line.find("resources") != 0)
        throw "Invalid resources file";
    
    unsigned int i = line.find("v");
    
    if(i == string::npos)
        throw "Unable to find version number in resources file";
    
    int ver = 0;
    
    istringstream verss(line.substr(i));
    
    verss.ignore();
    
    verss >> ver;
    
    if(ver != 1)
        throw "Invalid version number for resources file";
    
    while(in.good()) {
        
        getline(in, line);
        
        istringstream data(line);
        
        unsigned int id = 0;
        
        data >> id;
        
        if(id) {
            
            Resource &resource = resources[id];
            
            // Read in all data (data is only for this one line)
            data >> resource.type;
            data >> resource.x;
            data >> resource.y;
            data >> resource.radius;
            data >> resource.amount;
            data >> resource.building;
        }
    }
    
    // Recalculate the terminal links
    for(map<unsigned int, Terminal>::iterator i = terminals.begin(); i != terminals.end(); i++)
       i->second.recalculate_tracks();
    
    out << "Loaded resources, resources: " << resources.size() << "\n";
}

void World::saveResources(const string &filename) const
{
    ofstream file(filename.c_str());
    
    file << "resources v1\n";
    
    std::map<unsigned int, Resource>::const_iterator itr;
    
    for(itr = resources.begin(); itr != resources.end(); ++itr)
        file << itr->first << " "
             << itr->second.type << " "
             << itr->second.x << " "
             << itr->second.y << " "
             << itr->second.radius << " "
             << itr->second.building << " "
             << itr->second.amount << "\n";
    
    out << "Saved " << filename << " count: " << resources.size() << "\n";
}

void World::loadBuildings(const string &filename)
{
    buildings.clear();
    
    string line;
    
    ifstream in(filename.c_str());
    
    getline(in, line);
    
    if(line.find("buildings") != 0)
        throw "Invalid buildings file";
    
    unsigned int i = line.find("v");
    
    if(i == string::npos)
        throw "Unable to find version number in buildings file";
    
    int ver = 0;
    
    istringstream verss(line.substr(i));
    
    verss.ignore();
    
    verss >> ver;
    
    if(ver != 1)
        throw "Invalid version number for buildings file";
    
    while(in.good()) {
        
        getline(in, line);
        
        istringstream data(line);
        
        unsigned int id = 0;
        
        data >> id;
        
        if(id) {
            
            Building &building = buildings[id];
            
            // Read in all data (data is only for this one line)
            data >> building.x;
            data >> building.y;
            data >> building.type;
            data >> building.player;
            data >> building.track;
        }
    }
    
    out << "Loaded buildings, buildings: " << buildings.size() << "\n";
}

void World::saveBuildings(const string &filename) const
{
    ofstream file(filename.c_str());
    
    file << "buildings v1\n";
    
    std::map<unsigned int, Building>::const_iterator itr;
    
    for(itr = buildings.begin(); itr != buildings.end(); ++itr)
        file << itr->first << " "
             << itr->second.x << " "
             << itr->second.y << " "
             << itr->second.type << " "
             << itr->second.player << " "
             << itr->second.track << "\n";
    
    out << "Saved " << filename << " count: " << buildings.size() << "\n";
}

void World::loadTerminals(const string &filename)
{
    terminals.clear();
    
    string line;
    
    ifstream in(filename.c_str());
    
    getline(in, line);
    
    if(line.find("terminals") != 0)
        throw "Invalid terminals file";
    
    unsigned int i = line.find("v");
    
    if(i == string::npos)
        throw "Unable to find version number in terminals file";
    
    int ver = 0;
    
    istringstream verss(line.substr(i));
    
    verss.ignore();
    
    verss >> ver;
    
    if(ver != 1)
        throw "Invalid version number for terminals file";
    
    while(in.good()) {
        
        getline(in, line);
        
        istringstream data(line);
        
        unsigned int id = 0;
        
        data >> id;
        
        if(id) {
            
            Terminal &terminal = terminals[id];
            
            // Read in all data (data is only for this one line)
            data >> terminal.owner_id;
            data >> terminal.city;
            data >> terminal.track;
        }
    }
    
    // Recalculate the terminal links
    for(map<unsigned int, Terminal>::iterator i = terminals.begin(); i != terminals.end(); i++)
       i->second.recalculate_tracks();
    
    out << "Loaded terminals, terminals: " << buildings.size() << "\n";
}

void World::saveTerminals(const string &filename) const
{
    ofstream file(filename.c_str());
    
    file << "terminals v1\n";
    
    std::map<unsigned int, Terminal>::const_iterator itr;
    
    for(itr = terminals.begin(); itr != terminals.end(); ++itr)
        file << itr->first << " "
             << itr->second.owner_id << " "
             << itr->second.city << " "
             << itr->second.track << "\n";
    
    out << "Saved " << filename << " count: " << terminals.size() << "\n";
}

void World::loadTracks(const string &filename)
{
    tracks.clear();
    
    string line;
    
    ifstream in(filename.c_str());
    
    getline(in, line);
    
    if(line.find("tracks") != 0)
        throw "Invalid tracks file";
    
    unsigned int i = line.find("v");
    
    if(i == string::npos)
        throw "Unable to find version number in tracks file";
    
    int ver = 0;
    
    istringstream verss(line.substr(i));
    
    verss.ignore();
    
    verss >> ver;
    
    if(ver != 1)
        throw "Invalid version number for tracks file";
    
    while(in.good()) {
        
        getline(in, line);
        
        istringstream data(line);
        
        unsigned int id = 0;
        
        data >> id;
        
        if(id) {
            
            Track &track = tracks[id];
            
            // Read in all data (data is only for this one line)
            data >> track.player;
            data >> track.x;
            data >> track.y;
            data >> track.buildings[0];
            data >> track.buildings[1];
            
            vector<unsigned int> adj;
            track.find_adjacent(adj);
        }
    }
    
    // Recalculate the terminal links
    for(map<unsigned int, Terminal>::iterator i = terminals.begin(); i != terminals.end(); i++)
       i->second.recalculate_tracks();
    
    out << "Loaded tracks, tracks: " << tracks.size() << "\n";
}

void World::saveTracks(const string &filename) const
{
    ofstream file(filename.c_str());
    
    file << "tracks v1\n";
    
    std::map<unsigned int, Track>::const_iterator itr;
    
    for(itr = tracks.begin(); itr != tracks.end(); ++itr)
        file << itr->first << " "
             << itr->second.player << " "
             << itr->second.x << " "
             << itr->second.y << " "
             << itr->second.buildings[0] << " "
             << itr->second.buildings[1] << "\n";
    
    out << "Saved " << filename << " count: " << tracks.size() << "\n";
}

void World::loadTrains(const string &filename)
{
    trains.clear();
    
    string line;
    
    ifstream in(filename.c_str());
    
    getline(in, line);
    
    if(line.find("trains") != 0)
        throw "Invalid trains file";
    
    unsigned int i = line.find("v");
    
    if(i == string::npos)
        throw "Unable to find version number in trains file";
    
    int ver = 0;
    
    istringstream verss(line.substr(i));
    
    verss.ignore();
    
    verss >> ver;
    
    if(ver != 1)
        throw "Invalid version number for trains file";
    
    while(in.good()) {
        
        getline(in, line);
        
        istringstream data(line);
        
        unsigned int id = 0;
        
        data >> id;
        
        if(id) {
            
            Train &train = trains[id];
            
            // Read in all data (data is only for this one line)
            data >> train.type;
            data >> train.track;
            data >> train.player;
            data >> train.parentCar;
            data >> train.childCar;
        }
    }
    
    out << "Loaded trains, trains: " << trains.size() << "\n";
}

void World::saveTrains(const string &filename) const
{
    ofstream file(filename.c_str());
    
    file << "trains v1\n";
    
    std::map<unsigned int, Train>::const_iterator itr;
    
    for(itr = trains.begin(); itr != trains.end(); ++itr)
        file << itr->first << " "
             << itr->second.type << " "
             << itr->second.track << " "
             << itr->second.player << " "
             << itr->second.parentCar << " "
             << itr->second.childCar << "\n";
    
    out << "Saved " << filename << " count: " << trains.size() << "\n";
}

void World::loadUsers(const string &filename)
{
    users.clear();
    
    string line;
    
    ifstream in(filename.c_str());
    
    getline(in, line);
    
    if(line.find("users") != 0)
        throw "Invalid users file";
    
    unsigned int i = line.find("v");
    
    if(i == string::npos)
        throw "Unable to find version number in users file";
    
    int ver = 0;
    
    istringstream verss(line.substr(i));
    
    verss.ignore();
    
    verss >> ver;
    
    if(ver != 1)
        throw "Invalid version number for users file";
    
    while(in.good()) {
        
        getline(in, line);
        
        istringstream data(line);
        
        unsigned int id = 0;
        
        data >> id;
        
        if(id) {
            
            User &user = users[id];
            
            // Read in all data (data is only for this one line)
            data >> user.x;
            data >> user.y;
            data >> user.username;
            data >> user.password;
            
            user.items.clear();
            
            while(data.good()) {
                
                unsigned int id = 0;
                int count = 0;
                
                data >> id;
                data >> count;
                
                if(id)
                    user.items[id] += count;
            }
        }
    }
    
    out << "Loaded users, users: " << users.size() << "\n";
}

void World::saveUsers(const string &filename) const
{
    ofstream file(filename.c_str());
    
    file << "users v1\n";
    
    std::map<unsigned int, User>::const_iterator itr;
    
    for(itr = users.begin(); itr != users.end(); ++itr) {
        
        file << itr->first << " "
             << itr->second.x << " "
             << itr->second.y << " "
             << itr->second.username << " "
             << itr->second.password;
        
        User::Items::const_iterator jtr;
        
        for(jtr = itr->second.items.begin(); jtr != itr->second.items.end(); jtr++)
            file << " " << jtr->first << " " << jtr->second;
        
        file << "\n";
    }
    
    out << "Saved " << filename << " count: " << users.size() << "\n";
}
